#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.invokeCli = exports.invokeCommand = void 0;
const tslib_1 = require("tslib");
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
const yargsParser = require("yargs-parser");
const argv = yargsParser(process.argv.slice(2), {
    configuration: {
        'strip-dashed': true,
        'dot-notation': false,
    },
});
function invokeCommand(command, root, commandArgs = []) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        if (!command) {
            command = 'help';
        }
        let verboseFlagIndex = commandArgs.indexOf('--verbose');
        if (verboseFlagIndex < 0) {
            verboseFlagIndex = commandArgs.indexOf('-v');
        }
        const isVerbose = verboseFlagIndex >= 0;
        if (isVerbose) {
            commandArgs.splice(verboseFlagIndex, 1);
        }
        switch (command) {
            case 'new':
                return (yield Promise.resolve().then(() => require('../commands/generate'))).newWorkspace(root, commandArgs, isVerbose);
            case 'generate':
            case 'g':
                return (yield Promise.resolve().then(() => require('../commands/generate'))).generate(process.cwd(), root, commandArgs, isVerbose);
            case 'run':
            case 'r':
                return (yield Promise.resolve().then(() => require('../commands/run'))).run(process.cwd(), root, commandArgs, isVerbose);
            case '_migrate':
                return (yield Promise.resolve().then(() => require('../commands/migrate'))).migrate(root, commandArgs, isVerbose);
            case 'help':
            case '--help':
                return (yield Promise.resolve().then(() => require('../commands/help'))).help();
            default: {
                const projectNameIncluded = commandArgs[0] && !commandArgs[0].startsWith('-');
                const projectName = projectNameIncluded ? commandArgs[0] : '';
                // this is to make `nx test mylib` same as `nx run mylib:test`
                return (yield Promise.resolve().then(() => require('../commands/run'))).run(process.cwd(), root, [
                    `${projectName}:${command}`,
                    ...(projectNameIncluded ? commandArgs.slice(1) : commandArgs),
                ], isVerbose);
            }
        }
    });
}
exports.invokeCommand = invokeCommand;
function findWorkspaceRoot(dir) {
    if ((0, path_1.dirname)(dir) === dir) {
        throw new Error(`The cwd isn't part of an Nx workspace`);
    }
    if ((0, fs_extra_1.existsSync)((0, path_1.join)(dir, 'angular.json')) ||
        (0, fs_extra_1.existsSync)((0, path_1.join)(dir, 'workspace.json')) ||
        (0, fs_extra_1.existsSync)((0, path_1.join)(dir, 'nx.json'))) {
        return dir;
    }
    return findWorkspaceRoot((0, path_1.dirname)(dir));
}
function invokeCli(root, args) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        const [command, ...commandArgs] = args;
        process.exit(yield invokeCommand(command, root, commandArgs));
    });
}
exports.invokeCli = invokeCli;
invokeCli(argv.nxWorkspaceRoot || findWorkspaceRoot(process.cwd()), process.argv.slice(2));
//# sourceMappingURL=index.js.map